@@ -1,3 +1,4 @@
+# Lint as: python2, python3
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -30,16 +31,20 @@ nomenclature below will indicate "task data" even for such jobs that do not have
 explicit tasks.
 """
 
-from abc import ABCMeta
-from abc import abstractmethod
+import abc
+import six
 
 
+@six.add_metaclass(abc.ABCMeta)
 class JobProvider(object):
   """Interface all job providers should inherit from."""
-  __metaclass__ = ABCMeta
 
-  @abstractmethod
-  def prepare_job_metadata(self, script, job_name, user_id):
+  # A member field that a provider can use to expose a status message
+  # such as a deprecation notice.
+  status_message = None
+
+  @abc.abstractmethod
+  def prepare_job_metadata(self, script, job_name, user_id, create_time):
     """Returns a dictionary of metadata fields for the job.
 
     Call this before calling submit_job.
@@ -70,60 +75,58 @@ class JobProvider(object):
 
     Args:
       script: path to the job script
-      job_name: ?
+      job_name: user-supplied job name, if any
       user_id: user whose jobs to look for
+      create_time: create time for the job
     """
     raise NotImplementedError()
 
-  @abstractmethod
-  def submit_job(self, job_resources, job_metadata, job_data, all_task_data):
+  @abc.abstractmethod
+  def submit_job(self, job_descriptor, skip_if_output_present):
     """Submit the job to be executed.
 
     Args:
-      job_resources: resource parameters required by each job.
-      job_metadata: job parameters such as job-id, user-id, script.
-      job_data: job parameters included in each task.
-      all_task_data: list of parameters to launch each job task.
+      job_descriptor (job_model.JobDescriptor): parameters needed to launch all
+      job tasks
+      skip_if_output_present: (boolean) if true, skip tasks whose output
+        is present (see --skip flag for more explanation).
 
-    job_resources contains settings related to how many resources to give each
-    task. Its fields include: min_cores, min_ram, disk_size, boot_disk_size,
-    preemptible, image, zones.
-
-    job_metadata is a dictionary. Its fields include: 'script', 'pipeline-name',
-    'job-name', 'job-id', 'user-id', 'is_table'.
-
-    task_parameters is a list of dictionaries, one per task to execute.
-    Each contains the following fields: 'envs', 'inputs', 'outputs'.
 
     Returns:
       A dictionary containing the 'user-id', 'job-id', and 'task-id' list.
       For jobs that are not task array jobs, the task-id list should be empty.
+      If all tasks were skipped, then the job-id is dsub_lib.NO_JOB.
+
 
     Raises:
-      ValueError: submit job may validate any of the parameters and raise
+      ValueError: submit_job may validate any of the parameters and raise
         a value error if any parameter (or specific combination of parameters)
         is not supported by the provider.
     """
     raise NotImplementedError()
 
-  @abstractmethod
+  @abc.abstractmethod
   def delete_jobs(self,
-                  user_list,
-                  job_list,
-                  task_list,
+                  user_ids,
+                  job_ids,
+                  task_ids,
                   labels,
-                  create_time=None):
+                  create_time_min=None,
+                  create_time_max=None):
     """Kills the operations associated with the specified job or job.task.
 
     Some providers may provide only a "cancel" operation, which terminates the
     task but does not truly "delete" it from the "task list".
 
     Args:
-      user_list: List of user ids who "own" the job(s) to delete.
-      job_list: List of job ids to delete.
-      task_list: List of task ids to delete.
-      labels: List of LabelParam, each must match the job(s) to be cancelled.
-      create_time: a UTC value for earliest create time for a task.
+      user_ids: a set of user ids who "own" the job(s) to delete.
+      job_ids: a set of job ids to delete.
+      task_ids: a set of task ids to delete.
+      labels: a set of LabelParam, each must match the job(s) to be cancelled.
+      create_time_min: a timezone-aware datetime value for the earliest create
+                       time of a task, inclusive.
+      create_time_max: a timezone-aware datetime value for the most recent
+                       create time of a task, inclusive.
 
     Returns:
       (list of tasks canceled,
@@ -133,32 +136,44 @@ class JobProvider(object):
     """
     raise NotImplementedError()
 
-  @abstractmethod
+  @abc.abstractmethod
   def lookup_job_tasks(self,
-                       status_list,
-                       user_list=None,
-                       job_list=None,
-                       job_name_list=None,
-                       task_list=None,
+                       statuses,
+                       user_ids=None,
+                       job_ids=None,
+                       job_names=None,
+                       task_ids=None,
+                       task_attempts=None,
                        labels=None,
-                       create_time=None,
-                       max_tasks=0):
+                       create_time_min=None,
+                       create_time_max=None,
+                       max_tasks=0,
+                       page_size=0,
+                       verbose=True):
     """Return a list of tasks based on the search criteria.
 
-    If any of the filters are empty or "[*]", then no filtering is performed on
+    If any of the filters are empty or {'*'}, then no filtering is performed on
     that field. Filtering by both a job id list and job name list is
     unsupported.
 
     Args:
-      status_list: ['*'], or a list of job status strings to return. Valid
+      statuses: {'*'}, or a set of job status strings to return. Valid
         status strings are 'RUNNING', 'SUCCESS', 'FAILURE', or 'CANCELED'.
-      user_list: a list of ids for the user(s) who launched the job.
-      job_list: a list of job ids to return.
-      job_name_list: a list of job names to return.
-      task_list: a list of specific tasks within the specified job(s) to return.
+      user_ids: a set of ids for the user(s) who launched the job.
+      job_ids: a set of job ids to return.
+      job_names: a set of job names to return.
+      task_ids: a set of specific tasks within the specified job(s) to return.
+      task_attempts: a set of specific task attempts within the specified
+        tasks(s) to return.
       labels: a list of LabelParam, each must match the job(s) returned.
-      create_time: a UTC value for earliest create time for a task.
+      create_time_min: a timezone-aware datetime value for the earliest create
+                       time of a task, inclusive.
+      create_time_max: a timezone-aware datetime value for the most recent
+                       create time of a task, inclusive.
       max_tasks: the maximum number of job tasks to return or 0 for no limit.
+      page_size: the page size to use for each query to the backend. May be
+                 ignored by provider implementations.
+      verbose: if set to true, will output retrying error messages
 
     Returns:
       A list of Task objects.
@@ -168,7 +183,7 @@ class JobProvider(object):
     """
     raise NotImplementedError()
 
-  @abstractmethod
+  @abc.abstractmethod
   def get_tasks_completion_messages(self, tasks):
     """List of the error message of each given task."""
     raise NotImplementedError()
@@ -177,7 +192,7 @@ class JobProvider(object):
 class Task(object):
   """Basic container for task metadata."""
 
-  @abstractmethod
+  @abc.abstractmethod
   def raw_task_data(self):
     """Return a provider-specific representation of task data.
 
@@ -186,15 +201,16 @@ class Task(object):
     """
     raise NotImplementedError()
 
-  @abstractmethod
+  @abc.abstractmethod
   def get_field(self, field, default=None):
     """Return a metadata-field for the task.
 
     Not all fields need to be supported by all providers.
     Field identifiers include:
 
-    'job-name', 'job-id', 'task-id', 'user-id', 'task-status', 'error-message',
-    'create-time', 'start-time', 'end-time', 'inputs', 'outputs'
+    'job-name', 'job-id', 'task-id', 'task-attempt', 'user-id', 'task-status',
+    'error-message', 'create-time', 'start-time', 'end-time', 'inputs',
+    'outputs', 'events'
 
     The following are required by dstat:
     - status: The task status ('RUNNING', 'CANCELED', 'FAILED', 'SUCCESS')
